from .button import *  # noqa
from .icon import *  # noqa
from .indicators import *  # noqa
from .input import *  # noqa
from .menus import *  # noqa
from .misc import *  # noqa
from .select import *  # noqa
from .slider import *  # noqa



from __future__ import annotations

from typing import Callable

import param

from ..base import ThemedTransform
from ..widgets import TextAreaInput


class ChatAreaInput(TextAreaInput):
    """
    The `ChatAreaInput` allows entering any multiline string using a text input
    box, with the ability to press enter to submit the message.

    Unlike TextAreaInput, the `ChatAreaInput` defaults to auto_grow=True and
    max_rows=10, and the value is not synced to the server until the enter key
    is pressed so bind on `value_input` if you need to access the existing value.

    Lines are joined with the newline character `\\n`.

    :References:

    - https://panel-material-ui.holoviz.org/reference/chat/ChatAreaInput.html
    - https://panel.holoviz.org/reference/chat/ChatAreaInput.html

    :Example:

    >>> ChatAreaInput(max_rows=10)
    """

    actions = param.Dict(default={}, doc="""
        A dictionary of actions that can be invoked via the speed dial to the
        left of input area. The actions should be defined as a dictionary indexed
        by the name of the action mapping to values that themselves are dictionaries
        containing an icon. Users can define callbacks by registering callbacks using
        the on_action method.""")

    auto_grow = param.Boolean(default=True)

    disabled_enter = param.Boolean(
        default=False,
        doc="If True, disables sending the message by pressing the `enter_sends` key.",
    )

    enter_sends = param.Boolean(
        default=True,
        doc="If True, pressing the Enter key sends the message, if False it is sent by pressing the Ctrl+Enter.",
    )

    enter_pressed = param.Event(
        default=False,
        doc="If True, pressing the Enter key sends the message, if False it is sent by pressing the Ctrl+Enter.",
    )

    loading = param.Boolean(default=False)

    max_rows = param.Integer(default=10)

    max_length = param.Integer(default=50000, doc="""
        Max count of characters in the input field.""")

    rows = param.Integer(default=1)

    _esm_base = "ChatArea.jsx"

    _esm_transforms = [ThemedTransform]

    _rename = {"loading": "loading"}

    def __init__(self, **params):
        action_callbacks = {}
        if 'actions' in params:
            actions = {}
            for action, value in params['actions'].items():
                value = dict(value)
                if 'callback' in value:
                    action_callbacks[action] = value.pop('callback')
                actions[action] = value
            params['actions'] = actions
        super().__init__(**params)
        self._action_callbacks = {}
        for action, callback in action_callbacks.items():
            self.on_action(action, callback)

    def _handle_msg(self, msg) -> None:
        if msg['type'] == 'input':
            self.value = msg['value']
            self.param.trigger('enter_pressed')
            with param.discard_events(self):
                self.value = ""
        elif msg['type'] == 'action':
            for callback in self._action_callbacks.get(msg['action'], []):
                try:
                    callback(msg)
                except Exception:
                    pass

    def on_action(self, name: str, callback: Callable):
        """
        Allows registering a callback invoked when an action is defined.

        Parameters
        ----------
        name: str
            The name of the action to register the callback for.
        callback: callable
            The callback to invoke when the action is triggered.
        """
        if name not in self._action_callbacks:
            self._action_callbacks[name] = []
        self._action_callbacks[name].append(callback)

    def _update_loading(self, *_) -> None:
        """
        Loading handler handled client-side.
        """


__all__ = ["ChatAreaInput"]



from __future__ import annotations

from collections.abc import Callable
from typing import Any

import param
from panel.links import Callback

from ..base import COLORS, ThemedTransform
from .base import MaterialWidget, TooltipTransform
from .button import _ButtonBase


class _ClickableIcon(MaterialWidget):

    active_icon = param.String(
        default="",
        doc="""
        The name of the icon to display when toggled from
        https://mui.com/material-ui/material-icons or an SVG.""",
    )

    icon = param.String(
        default="heart",
        doc="""
        The name of the icon to display from
        https://mui.com/material-ui/material-icons or an SVG.""",
    )

    size = param.Selector(objects=["small", "medium", "large"], default="medium")

    value = param.Boolean(
        default=False,
        doc="""
        Whether the icon is toggled on or off.""",
    )

    _esm_transforms = [TooltipTransform, ThemedTransform]

    __abstract = True


class ToggleIcon(_ClickableIcon):
    """
    The `ToggleIcon` widget allows toggling a single condition between True/False states. This
    widget is interchangeable with the `Checkbox` and `Switch` widget.

    This widget incorporates a `value` attribute, which alternates between `False` and `True`.

    :References:

    - https://panel-material-ui.holoviz.org/reference/widgets/ToggleIcon.html
    - https://panel.holoviz.org/reference/widgets/ToggleIcon.html
    - https://mui.com/material-ui/react-checkbox/#icon

    :Example:

    >>> ToggleIcon(
    ...     icon="thumb-up", active_icon="thumb-down", size="small", description="Like"
    ... )
    """

    color = param.Selector(objects=COLORS, default="default")

    description_delay = param.Integer(default=1000, doc="""
        Delay (in milliseconds) to display the tooltip after the cursor has
        hovered over the Button, default is 1000ms.""")

    width = param.Boolean(default=None)

    _esm_base = "ToggleIcon.jsx"


class IconButton(_ClickableIcon, _ButtonBase):
    """
    The `IconButton` widget facilitates event triggering upon button clicks.

    This widget displays a default `icon` initially. Upon being clicked, an `active_icon` appears
    for a specified `toggle_duration`.

    For instance, the `IconButton` can be effectively utilized to implement a feature akin to
    ChatGPT's copy-to-clipboard button.

    The button incorporates a `value` attribute, which alternates between `False` and `True` as the
    click event is processed.

    Furthermore, it includes an `clicks` attribute, enabling subscription to click events for
    further actions or monitoring.

    :References:

    - https://panel-material-ui.holoviz.org/reference/widgets/IconButton.html
    - https://panel.holoviz.org/reference/widgets/ButtonIcon.html
    - https://mui.com/material-ui/api/icon-button/

    :Example:

    >>> button_icon = IconButton(
    ...     icon='favorite',
    ...     active_icon='check',
    ...     description='Copy',
    ...     toggle_duration=2000
    ... )
    """

    clicks = param.Integer(default=0, doc="""
        The number of times the button has been clicked.""")

    edge = param.Selector(objects=["start", "end", False], default=False, doc="""
        Whether the icon should be on the start or end of the button.""")

    size = param.String(default="1em", doc="""
        Size of the icon as a string, e.g. 12px or 1em.""",)

    toggle_duration = param.Integer(default=75, doc="""
        The number of milliseconds the active_icon should be shown for
        and how long the button should be disabled for.""")

    value = param.Event(doc="Toggles from False to True while the event is being processed.")

    _esm_base = "IconButton.jsx"
    _event = "dom_event"

    def __init__(self, **params):
        click_handler = params.pop('on_click', None)
        super().__init__(**params)
        if click_handler:
            self.on_click(click_handler)

    def _handle_click(self, event):
        self.param.update(clicks=self.clicks + 1, value=True)

    def on_click(
        self, callback: Callable[[param.parameterized.Event], None]
    ) -> param.parameterized.Watcher:
        """
        Register a callback to be executed when the button is clicked.

        The callback is given an `Event` argument declaring the number of clicks.

        Arguments
        ---------
        callback: (Callable[[param.parameterized.Event], None])
            The function to run on click events. Must accept a positional `Event` argument

        Returns
        -------
        watcher: param.Parameterized.Watcher
            A `Watcher` that executes the callback when the MenuButton is clicked.
        """
        return self.param.watch(callback, 'clicks', onlychanged=False)

    def js_on_click(self, args: dict[str, Any] | None = None, code: str = "") -> Callback:
        """
        Allows defining a JS callback to be triggered when the button
        is clicked.

        Parameters
        -----------
        args: dict
          A mapping of objects to make available to the JS callback
        code: str
          The Javascript code to execute when the button is clicked.

        Returns
        -------
        callback: Callback
          The Callback which can be used to disable the callback.
        """
        if args is None:
            args = {}
        return Callback(self, code={'event:'+self._event: code}, args=args)

    def jscallback(self, args: dict[str, Any] | None = None, **callbacks: str) -> Callback:
        """
        Allows defining a Javascript (JS) callback to be triggered when a property
        changes on the source object. The keyword arguments define the
        properties that trigger a callback and the JS code that gets
        executed.

        Parameters
        -----------
        args: dict
          A mapping of objects to make available to the JS callback
        **callbacks: dict
          A mapping between properties on the source model and the code
          to execute when that property changes

        Returns
        -------
        callback: Callback
          The Callback which can be used to disable the callback.
        """
        if args is None:
            args = {}
        for k, v in list(callbacks.items()):
            if k == 'clicks':
                k = 'event:'+self._event
            val = self._rename.get(v, v)
            if val is not None:
                callbacks[k] = val
        return Callback(self, code=callbacks, args=args)


ButtonIcon = IconButton

__all__ = [
    "ButtonIcon",
    "IconButton",
    "ToggleIcon"
]



"""
A module containing testing utilities and fixtures.
"""
import asyncio
import os
import re
import socket
import unittest

from functools import cache

import pytest

from bokeh.document import Document
from bokeh.io.doc import curdoc, set_curdoc as set_bkdoc
from pyviz_comms import Comm

from panel import config, serve
from panel.io.state import set_curdoc, state
from panel.tests.util import get_open_ports

config.apply_signatures = False

if os.name != 'nt':
    import resource

    soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
    resource.setrlimit(resource.RLIMIT_NOFILE, (hard, hard))


for e in os.environ:
    if e.startswith(('BOKEH_', "PANEL_")) and e not in ("PANEL_LOG_LEVEL", ):
        os.environ.pop(e, None)

try:
    asyncio.get_event_loop()
except (RuntimeError, DeprecationWarning):
    asyncio.set_event_loop(asyncio.new_event_loop())

@cache
def internet_available(host="8.8.8.8", port=53, timeout=3):
    """Check if the internet connection is available."""
    try:
        socket.setdefaulttimeout(timeout)
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as conn:
            conn.connect((host, port))
        return True
    except socket.error:
        return False

def port_open(port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(1)
    is_open = sock.connect_ex(("127.0.0.1", port)) == 0
    sock.close()
    return is_open


def get_default_port():
    worker_count = int(os.environ.get("PYTEST_XDIST_WORKER_COUNT", "1"))
    worker_id = os.environ.get("PYTEST_XDIST_WORKER", "0")
    worker_idx = int(re.sub(r"\D", "", worker_id))
    return 9001 + (worker_idx * worker_count * 10)

optional_markers = {
    "ui": {
        "help": "Runs UI related tests",
        "marker-descr": "UI test marker",
        "skip-reason": "Test only runs with the --ui option."
    },
}


def pytest_addoption(parser):
    for marker, info in optional_markers.items():
        parser.addoption(f"--{marker}", action="store_true",
                         default=False, help=info['help'])
    parser.addoption('--repeat', action='store',
        help='Number of times to repeat each test')


def pytest_generate_tests(metafunc):
    repeat = getattr(metafunc.config.option, 'repeat', None)
    if repeat is not None:
        count = int(repeat)

        # We're going to duplicate these tests by parametrizing them,
        # which requires that each test has a fixture to accept the parameter.
        # We can add a new fixture like so:
        metafunc.fixturenames.append('tmp_ct')

        # Now we parametrize. This is what happens when we do e.g.,
        # @pytest.mark.parametrize('tmp_ct', range(count))
        # def test_foo(): pass
        metafunc.parametrize('tmp_ct', range(count))

def pytest_collection_modifyitems(config, items):
    skipped, selected = [], []
    markers = [m for m in optional_markers if config.getoption(f"--{m}")]
    empty = not markers
    for item in items:
        if empty and any(m in item.keywords for m in optional_markers):
            skipped.append(item)
        elif empty:
            selected.append(item)
        elif not empty and any(m in item.keywords for m in markers):
            selected.append(item)
        else:
            skipped.append(item)

    config.hook.pytest_deselected(items=skipped)
    items[:] = selected


def pytest_runtest_setup(item):
    if "internet" in item.keywords and not internet_available():
        pytest.skip("Skipping test: No internet connection")


@pytest.fixture
def context(context):
    # Set the default timeout to 20 secs
    context.set_default_timeout(20_000)
    yield context

PORT = [get_default_port()]

@pytest.fixture
def document():
    return Document()

@pytest.fixture
def server_document():
    doc = Document()
    session_context = unittest.mock.Mock()
    doc._session_context = lambda: session_context
    try:
        with set_curdoc(doc):
            yield doc
    finally:
        doc._session_context = None

@pytest.fixture
def bokeh_curdoc():
    old_doc = curdoc()
    doc = Document()
    session_context = unittest.mock.Mock()
    doc._session_context = lambda: session_context
    set_bkdoc(doc)
    try:
        yield doc
    finally:
        set_bkdoc(old_doc)

@pytest.fixture
def comm():
    return Comm()


@pytest.fixture
def port():
    return get_open_ports()[0]



import datetime

import pytest

pytest.importorskip('playwright')

from panel.tests.util import serve_component, wait_until
from panel_material_ui.widgets import DatetimePicker
from playwright.sync_api import expect

pytestmark = pytest.mark.ui


def test_datetime_picker(page):
    """Test basic functionality of DatetimePicker."""
    datetime_picker = DatetimePicker(value=datetime.datetime(2023, 6, 15, 14, 30))

    serve_component(page, datetime_picker)

    # Check if the component is rendered
    expect(page.locator(".MuiInputBase-root")).to_have_count(1)

    # We need to wait for the component to fully initialize
    wait_until(lambda: "2023-06-15" in page.locator(".MuiInputBase-input").input_value(), page)

    # The datetime value in Python model should match the original value
    assert datetime_picker.value.year == 2023
    assert datetime_picker.value.month == 6
    assert datetime_picker.value.day == 15
    assert datetime_picker.value.hour == 14
    assert datetime_picker.value.minute == 30


def test_datetime_picker_with_string(page):
    """Test DatetimePicker with string datetime value."""
    datetime_picker = DatetimePicker(value="2023-06-15 14:30:00")

    serve_component(page, datetime_picker)

    # Wait for initialization
    wait_until(lambda: "2023-06-15" in page.locator(".MuiInputBase-input").input_value(), page)

    # The datetime value in Python model should be parsed correctly
    assert isinstance(datetime_picker.value, datetime.datetime)
    assert datetime_picker.value.year == 2023
    assert datetime_picker.value.month == 6
    assert datetime_picker.value.day == 15
    assert datetime_picker.value.hour == 14
    assert datetime_picker.value.minute == 30


@pytest.mark.parametrize('variant', ["filled", "outlined", "standard"])
def test_datetime_picker_variant(page, variant):
    """Test different variants of DatetimePicker."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        variant=variant
    )

    serve_component(page, datetime_picker)

    # Check if the component with the specific variant is rendered
    if variant == "standard":
        expect(page.locator(".MuiInput-root")).to_have_count(1)
    else:
        expect(page.locator(f".Mui{variant.capitalize()}Input-root")).to_have_count(1)


@pytest.mark.parametrize('color', ["primary", "secondary", "error", "info", "success", "warning"])
def test_datetime_picker_color(page, color):
    """Test different colors of DatetimePicker."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        color=color
    )

    serve_component(page, datetime_picker)

    # Check if the component is rendered
    expect(page.locator(".MuiInputBase-root")).to_have_count(1)


@pytest.mark.parametrize('military_time,expected_format', [
    (True, ["14:30"]),  # 24h format
    (False, ["02:30", "pm"])  # 12h format
])
def test_datetime_picker_time_format(page, military_time, expected_format):
    """Test 12h and 24h time formats."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        military_time=military_time
    )

    serve_component(page, datetime_picker)

    # Check rendering only for now
    expect(page.locator(".MuiInputBase-root")).to_have_count(1)


def test_datetime_picker_disabled(page):
    """Test disabled state of DatetimePicker."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        disabled=True
    )

    serve_component(page, datetime_picker)

    # Check if the component is disabled
    expect(page.locator(".MuiInputBase-root.Mui-disabled")).to_have_count(1)


def test_datetime_picker_min_max(page):
    """Test min and max date constraints."""
    # Set bounds from June 1 to June 30, 2023
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        start=datetime.datetime(2023, 6, 1),
        end=datetime.datetime(2023, 6, 30)
    )

    serve_component(page, datetime_picker)

    # Check if the component is rendered
    expect(page.locator(".MuiInputBase-root")).to_have_count(1)

    # Verify input value
    input_value = page.locator(".MuiInputBase-input").input_value()
    assert "2023-06-15" in input_value

    # The value should be valid
    assert datetime_picker.value is not None


def test_datetime_picker_with_seconds(page):
    """Test DatetimePicker with seconds enabled."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30, 45),
        enable_seconds=True
    )

    serve_component(page, datetime_picker)

    # Wait for initialization
    wait_until(lambda: "2023-06-15" in page.locator(".MuiInputBase-input").input_value(), page)

    # Verify the value includes seconds
    assert datetime_picker.value.second == 45


def test_datetime_picker_format_update(page):
    """Test format updates when settings change."""
    datetime_picker = DatetimePicker(
        value=datetime.datetime(2023, 6, 15, 14, 30),
        military_time=True
    )

    serve_component(page, datetime_picker)

    # Check rendering only for now
    expect(page.locator(".MuiInputBase-root")).to_have_count(1)



from __future__ import annotations

import param
from panel.widgets.misc import FileDownload as _FileDownload

from .button import _ButtonBase


class FileDownload(_ButtonBase, _FileDownload):
    """
    The `FileDownload` widget allows a user to download a file.

    It works either by sending the file data to the browser on initialization
    (`embed`=True), or when the button is clicked.

    :References:

    - https://panel-material-ui.holoviz.org/reference/widgets/FileDownload.html
    - https://panel.holoviz.org/reference/widgets/FileDownload.html
    - https://mui.com/material-ui/react-button/

    :Example:

    >>> FileDownload(file='IntroductionToPanel.ipynb', filename='intro.ipynb')
    """

    icon_size = param.String(default="1em", doc="""
        Size of the icon as a string, e.g. 12px or 1em.""")

    _esm_base = "FileDownload.jsx"
    _rename = {"_clicks": None, "icon": "icon", "icon_size": "icon_size"}
    _source_transforms = {"button_type": None, "button_style": None, "callback": None, "file": None, "value": None}

    def __init__(self, file=None, **params):
        self._default_label = 'label' not in params
        self._synced = False
        super().__init__(file=file, **params)

    def _handle_click(self, event=None):
        self._clicks += 1

__all__ = [
    "FileDownload"
]



from __future__ import annotations

from typing import TYPE_CHECKING, Callable

import param
from bokeh.models import Spacer as BkSpacer
from panel._param import Margin
from panel.layout.base import ListLike, NamedListLike, SizingModeMixin
from panel.viewable import Child

from ..base import COLORS, MaterialComponent
from ..widgets import ToggleIcon

if TYPE_CHECKING:
    from bokeh.document import Document
    from bokeh.model import Model
    from panel.viewable import Viewable
    from pyviz_comms import Comm


class MaterialLayout(MaterialComponent, SizingModeMixin):

    margin = Margin(default=0, doc="The margin of the layout.")

    __abstract = True

    def __init__(self, *objects, **params):
        if objects:
            params["objects"] = objects
        super().__init__(**params)

    def _get_model(
        self, doc: Document, root: Model | None = None,
        parent: Model | None = None, comm: Comm | None = None
    ) -> Model:
        model = super()._get_model(doc, root, parent, comm)
        props = dict(model.properties_with_values())
        props["sizing_mode"] = self.sizing_mode
        sizing_mode = self._compute_sizing_mode(model.data.objects, props)
        model.update(**sizing_mode)
        return model


class MaterialListLike(MaterialLayout, ListLike):

    __abstract = True


class MaterialNamedListLike(MaterialLayout, NamedListLike):

    _names = param.List(default=[])

    __abstract = True

    @param.depends("objects", watch=True)
    def _trigger_names(self):
        self.param.trigger("_names")


class Column(MaterialListLike):
    """
    The `Column` layout arranges its contents vertically.
    """

    _esm_base = "Box.jsx"
    _constants = {"direction": "column"}


class Row(MaterialListLike):
    """
    The `Row` layout arranges its contents horizontally.
    """

    _esm_base = "Box.jsx"
    _constants = {"direction": "row"}


class PaperMixin(param.Parameterized):
    """
    Baseclass adding Paper parameters to a layout.
    """

    elevation = param.Integer(default=1, bounds=(0, None))

    raised = param.Boolean(default=True, doc="Whether the layout is raised visually.")

    square = param.Boolean(default=False, doc="Whether to disable rounded corners.")

    variant = param.Selector(objects=["elevation", "outlined"], default="elevation")

    _abstract = True


class Paper(MaterialListLike, PaperMixin):
    """
    Paper implements a container for displaying content on an elevated surface.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Paper.html
    - https://mui.com/material-ui/react-paper/

    :Example:
    >>> Paper(name="Paper", objects=[1, 2, 3], elevation=10, width=200, height=200)
    """

    direction = param.Selector(objects=["row", "column", "column-reverse", "row-reverse"], default="column")

    _esm_base = "Paper.jsx"


class Container(MaterialListLike):
    """
    The `Container` layout centers your content horizontally. It's the most basic layout element.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Container.html
    - https://mui.com/material-ui/react-container/

    :Example:

    >>> Container(some_widget, some_pane, some_python_object, title='Container')
    """

    disable_gutters = param.Boolean(default=False, doc="""
        If `True`, the container will not have gutters.""")

    fixed = param.Boolean(default=False, doc="""
        Set the max-width to match the min-width of the current breakpoint.
        This is useful if you'd prefer to design for a fixed set of sizes
        instead of trying to accommodate a fully fluid viewport.""")

    sizing_mode = param.Selector(default='stretch_width')

    width_option = param.Selector(objects=["xs", "sm", "md", "lg", "xl", False], default="lg")

    _esm_base = "Container.jsx"


class Grid(MaterialListLike):
    """
    The `Grid` layout is a two-dimensional layout that allows arranging items in a grid.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Grid.html
    - https://mui.com/material-ui/react-grid/

    :Example:

    >>> Grid(some_widget, some_pane, some_python_object, title='Grid')
    """

    container = param.Boolean(default=False, doc="""
        Whether the grid should be a container.""")

    columns = param.ClassSelector(default=12, class_=(int, dict), doc="""
        The number of columns to display in the grid.""")

    column_spacing = param.Number(default=None, doc="""
        The spacing between the columns in the grid. Overrides the `spacing` parameter.""")

    direction = param.Selector(objects=["row", "column", "column-reverse", "row-reverse"], default="row")

    row_spacing = param.Number(default=None, doc="""
        The spacing between the rows in the grid. Overrides the `spacing` parameter.""")

    size = param.ClassSelector(default=None, class_=(int, str, dict), doc="""
        The size of the grid. Overrides the `columns` parameter.""")

    spacing = param.Number(default=0, doc="""
        The spacing between the columns and rows in the grid.""")

    _esm_base = "Grid.jsx"


class Card(MaterialNamedListLike, PaperMixin):
    """
    A `Card` layout allows arranging multiple panel objects in a
    collapsible, vertical container with a header bar.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Card.html
    - https://panel.holoviz.org/reference/layouts/Card.html
    - https://mui.com/material-ui/react-card/

    :Example:

    >>> Card(some_widget, some_pane, some_python_object, title='Card')
    """

    collapsed = param.Boolean(default=False, doc="""
        Whether the contents of the Card are collapsed.""")

    collapsible = param.Boolean(default=True, doc="""
        Whether the Card should be expandable and collapsible.""")

    header = Child(doc="""
        A Panel component to display in the header bar of the Card.
        Will override the given title if defined.""")

    header_background = param.Color(doc="""
        The background color of the Card header.""")

    header_color = param.Color(doc="""
        The text color of the Card header.""")

    header_css_classes = param.List(doc="""
        List of CSS classes to apply to CardHeader component.""")

    hide_header = param.Boolean(default=False)

    outlined = param.Boolean(default=True)

    title = param.String(doc="""
        A title to be displayed in the Card header, will be overridden
        by the header if defined.""")

    title_css_classes = param.List(doc="""
        List of CSS classes to apply to CardTitle component.""")

    _direction = "vertical"
    _esm_base = "Card.jsx"

    _rename = {"objects": "objects", "title": "title", "header": "header"}

    def select(self, selector: type | Callable[[Viewable], bool] | None = None) -> list[Viewable]:
        return ([] if self.header is None else self.header.select(selector)) + super().select(selector)


class Accordion(MaterialNamedListLike, PaperMixin):
    """
    The `Accordion` layout is a type of `Card` layout that allows switching
    between multiple objects by clicking on the corresponding card header.

    The labels for each card will default to the `name` parameter of the card's
    contents, but may also be defined explicitly as part of a tuple.

    `Accordion` has a list-like API that allows
    interactively updating and modifying the cards using the methods `append`,
    `extend`, `clear`, `insert`, `pop`, `remove` and `__setitem__`.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Accordion.html
    - https://panel.holoviz.org/reference/layouts/Accordion.html
    - https://mui.com/material-ui/react-accordion/

    :Example:

    >>> Accordion(("Card 1", "Card 1 objects"), ("Card 2", "Card 2 objects"))
    """

    active_header_color = param.Color(doc="""
        The text color of the active Card header.""")

    active_header_background = param.Color(doc="""
        The background color of the active Card header.""")

    active = param.List(default=[], doc="""
        List of indexes of active cards.""")

    disabled = param.List(default=[], doc="""
        List of indexes of disabled cards.""")

    disable_gutters = param.Boolean(default=False, doc="""
        Whether to disable margins between expanded sections.""")

    header_background = param.Color(doc="""
        The background color of the Card header.""")

    header_color = param.Color(doc="""
        The text color of the Card header.""")

    toggle = param.Boolean(default=False, doc="""
        Whether to toggle between active cards or allow multiple cards""")

    _esm_base = "Accordion.jsx"

    def __init__(self, *objects, **params):
        if "objects" not in params:
            params["objects"] = objects
        super().__init__(**params)


class Tabs(MaterialNamedListLike):
    """
    The `Tabs` layout allows switching between multiple objects by clicking
    on the corresponding tab header.

    Tab labels may be defined explicitly as part of a tuple or will be
    inferred from the `name` parameter of the tab's contents.

    Like `Accordion`, `Tabs` has a list-like API with methods to
    `append`, `extend`, `clear`, `insert`, `pop`, `remove` and `__setitem__`,
    which make it possible to interactively update and modify the tabs.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Tabs.html
    - https://panel.holoviz.org/reference/layouts/Tabs.html
    - https://mui.com/material-ui/react-tabs/

    :Example:

    >>> Tabs(("Tab 1", "Tab 1 objects"), ("Tab 2", "Card 2 objects"))

    """
    active = param.Integer(default=0, bounds=(0, None), doc="""
        Index of the currently displayed objects.""")

    closable = param.Boolean(default=False, doc="""
        Whether to display an icon to allow closing and thereby
        removing a tab.""")

    centered = param.Boolean(default=False, doc="""
        Whether the tabs should be centered.""")

    color = param.Selector(default="default", objects=["default", "primary", "secondary"])

    disabled = param.List(default=[], item_type=int, doc="""
        List of indexes of disabled tabs.""")

    dynamic = param.Boolean(default=False, doc="""
        Whether the tab contents should be rendered dynamically,
        i.e. only when the tab is active.""")

    tabs_location = param.ObjectSelector(
        default="above",
        objects=["above", "below", "left", "right"],
        doc="""
        The location of the tabs relative to the tab contents.""",
    )

    wrapped = param.Boolean(default=False, doc="""
        Whether the tab labels should be wrapped.""")

    _direction = "vertical"
    _esm_base = "Tabs.jsx"

    def __init__(self, *objects, **params):
        if "objects" not in params:
            params["objects"] = objects
        super().__init__(**params)

    @param.depends("active", watch=True)
    def _trigger_children(self):
        self.param.trigger("objects")

    def _get_child_model(self, child, doc, root, parent, comm):
        ref = root.ref["id"]
        models, old_models = [], []
        for i, sv in enumerate(child):
            if self.dynamic and i != self.active:
                model = BkSpacer()
            elif ref in sv._models:
                model = sv._models[ref][0]
                old_models.append(model)
            else:
                model = sv._get_model(doc, root, parent, comm)
            models.append(model)
        return models, old_models

    def _process_close(self, ref, attr, old, new):
        """
        Handle closed tabs.
        """
        model, _ = self._models.get(ref)
        if model:
            try:
                inds = [old.index(tab) for tab in new]
            except Exception:
                return old, None
            old = self.objects
            new = [old[i] for i in inds]
        return old, new

    def _comm_change(self, doc, ref, comm, subpath, attr, old, new):
        if attr in self._changing.get(ref, []):
            self._changing[ref].remove(attr)
            return
        if attr == 'objects':
            old, new = self._process_close(ref, attr, old, new)
            if new is None:
                return
        super()._comm_change(doc, ref, comm, subpath, attr, old, new)

    def _server_change(self, doc, ref, subpath, attr, old, new):
        if attr in self._changing.get(ref, []):
            self._changing[ref].remove(attr)
            return
        if attr == 'objects':
            old, new = self._process_close(ref, attr, old, new)
            if new is None:
                return
        super()._server_change(doc, ref, subpath, attr, old, new)


class Divider(MaterialListLike):
    """
    A `Divider` draws a horizontal rule (a `<hr>` tag in HTML) to separate
    multiple components in a layout.

    :References:

    - https://panel.holoviz.org/reference/layouts/Divider.html
    - https://mui.com/material-ui/react-divider/

    :Example:

    >>> Divider(sizing_mode="stretch_width")
    """

    orientation = param.Selector(default="horizontal", objects=["horizontal", "vertical"])

    variant = param.Selector(default="fullWidth", objects=["fullWidth", "inset", "middle"])

    _esm_base = "Divider.jsx"


class Alert(MaterialListLike):
    """
    An `Alert` displays messages, such as warnings, errors, success messages,
    or informational updates. It provides a visually distinct way to inform users about
    the system's status.

    :References:

    - https://panel-material-ui.holoviz.org/reference/global/Notifications.html
    - https://mui.com/material-ui/react-alert/

    :Example:

    >>> Alert(title="This is an alert")
    """

    alert_type = param.Selector(objects=COLORS, default="primary", doc="""
        The type of the alert.""")

    closed = param.Boolean(default=False, doc="""
        Whether the alert is closed.""")

    closeable = param.Boolean(default=False, doc="""
        Whether the alert is closeable.""")

    severity = param.Selector(objects=["error", "warning", "info", "success"], default="success", doc="""
        The severity of the alert.""")

    object = param.String(default="", doc="""
        The object to display in the alert.""")

    title = param.String(default=None, doc="""
        The title of the alert.""")

    variant = param.Selector(default="outlined", objects=["filled", "outlined"], doc="""
        The variant of the alert.""")

    _esm_base = "Alert.jsx"


class Backdrop(MaterialListLike):
    """
    The `Backdrop` component can be used to create a semi-transparent overlay over the application's UI.
    It is often used to focus attention on a specific part of the interface,
    such as during loading states or while a modal dialog is open.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Backdrop.html
    - https://mui.com/material-ui/react-backdrop/

    :Example:
    >>> close = Button(on_click=lambda _: backdrop.param.update(open=False), label='Close')  # type: ignore
    >>> backdrop = Backdrop(LoadingIndicator(), close)
    >>> button = Button(on_click=lambda _: backdrop.param.update(open=True), label=f'Open {Backdrop.name}')
    >>> pn.Column(button, backdrop).servable()
    """

    open = param.Boolean(default=False, doc="""
        Whether the backdrop is open.""")

    _esm_base = "Backdrop.jsx"


class Dialog(MaterialListLike):
    """
    The `Dialog` can be used to display important content in a modal-like overlay that requires
    user interaction. It is often used for tasks such as confirmations, forms, or displaying
    additional information.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Dialog.html
    - https://mui.com/material-ui/react-dialog/

    :Example:
    >>> close = Button(on_click=lambda _: dialog.param.update(open=False), label='Close')  # type: ignore
    >>> dialog = Dialog("This is a modal", close)
    >>> button = Button(on_click=lambda _: dialog.param.update(open=True), label=f'Open {Dialog.name}')
    >>> pn.Column(button, dialog).servable()
    """

    full_screen = param.Boolean(default=False, doc="""
        Whether the dialog should be full screen.""")

    open = param.Boolean(default=False, doc="""
        Whether the dialog is open.""")

    title = param.String(default="", doc="""
        The title of the dialog.""")

    scroll = param.Selector(objects=["body", "paper"], default="paper", doc="""
        Whether the dialog should scroll the content or the paper.""")

    width_option = param.Selector(objects=["xs", "sm", "md", "lg", "xl", False], default="sm", doc="""
        The width of the dialog.""")

    _esm_base = "Dialog.jsx"


class Drawer(MaterialListLike):
    """
    The `Drawer` component can be used to display important content in a modal-like overlay that requires
    user interaction. It is often used for tasks such as confirmations, forms, or displaying
    additional information.

    :References:

    - https://panel-material-ui.holoviz.org/reference/layouts/Drawer.html
    - https://mui.com/material-ui/react-drawer/

    :Example:
    >>> drawer = Drawer("This is a drawer")
    >>> button = Button(on_click=lambda _: drawer.param.update(open=True), label='Open Drawer')
    >>> pn.Column(button, drawer).servable()
    """

    anchor = param.Selector(default="left", objects=["left", "right", "top", "bottom"])

    size = param.Integer(default=250, doc="""
        The width (for left/right anchors) or height (for top/bottom anchors) of the drawer.""")

    open = param.Boolean(default=False, doc="""
        Whether the drawer is open.""")

    variant = param.Selector(default="temporary", objects=["permanent", "persistent", "temporary"])

    _esm_base = "Drawer.jsx"

    @param.depends("variant", watch=True, on_init=True)
    def _force_zero_dimensions(self):
        if self.variant == "temporary":
            self.param.update(width=0, height=0, sizing_mode="fixed")

    def create_toggle(
        self,
        icon: str = "menu",
        active_icon: str = "menu_open_icon",
        **params
    ):
        """
        Create a ToggleIcon for the drawer.

        Parameters
        ----------
        icon: str
            The icon to display when the drawer is closed.
        active_icon: str
            The icon to display when the drawer is open.

        Returns
        -------
        toggle: ToggleIcon
            A ToggleIcon component that can be used to toggle the drawer.
        """

        toggle = ToggleIcon(icon=icon, active_icon=active_icon, value=self.open, **params)
        toggle.jslink(self, value='open', bidirectional=True)
        return toggle


__all__ = [
    "Accordion",
    "Alert",
    "Backdrop",
    "Card",
    "Column",
    "Container",
    "Dialog",
    "Divider",
    "Drawer",
    "Grid",
    "Paper",
    "Row",
    "Tabs",
]



from __future__ import annotations

import uuid
from typing import TYPE_CHECKING, Any

import param
from bokeh.models.callbacks import CustomJS
from panel.io.datamodel import _DATA_MODELS, construct_data_model
from panel.io.notifications import Notification as _Notification
from panel.io.notifications import NotificationAreaBase
from panel.io.state import _state
from panel.layout import Column

from .base import MaterialComponent
from .widgets import Button, ColorPicker, NumberInput, Select, TextInput

if TYPE_CHECKING:
    from bokeh.document import Document
    from bokeh.models import Model
    from pyviz_comms import Comm


class MuiNotification(_Notification):

    _uuid = param.String(default=None)

    def __init__(self, **params):
        if '_uuid' not in params:
            params['_uuid'] = uuid.uuid4().hex
        super().__init__(**params)


class NotificationArea(MaterialComponent, NotificationAreaBase):

    notifications = param.List(item_type=(MuiNotification, dict))

    types = param.List(default=[], doc="""
        Custom notification types.

        Each type is a dictionary with the following keys:
        - 'type': The type of the notification.
        - 'background': The background color of the notification.
        - 'icon': The icon of the notification.
        """)

    _esm_base = "NotificationArea.jsx"

    _notification_type = MuiNotification

    _rename = {"max_notifications": "max_notifications", "anchor": "anchor"}

    _root_node = '#notistack-container'

    def __init__(self, **params):
        super().__init__(**params)
        self._notification_watchers = {}

    def _get_model(
        self, doc: Document, root: Model | None = None,
        parent: Model | None = None, comm: Comm | None = None
    ) -> Model:
        model = super()._get_model(doc, root, parent, comm)
        for event, notification in self.js_events.items():
            doc.js_on_event(event, CustomJS(code=f"""
            const config = {{
              message: {notification['message']!r},
              duration: {notification.get('duration', 0)},
              notification_type: {notification['type']!r},
              _destroyed: false,
              _uuid: Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15)
            }}
            notifications.document.event_manager.trigger(
              {{event_name: 'esm_event', model: notifications, data: {{type: 'enqueue', notification: config}}}}
            )
            notifications.data.notifications = [...notifications.data.notifications, config]
            """, args={'notifications': model}))
        return model

    def _process_events(self, events: dict[str, Any]) -> None:
        if 'notifications' in events:
            old = {n._uuid: n for n in self.notifications}
            notifications = []
            for notification in events.pop('notifications'):
                if isinstance(notification, dict):
                    notification = MuiNotification(notification_area=self, **notification)
                    self._notification_watchers[notification] = (
                        notification.param.watch(self._remove_notification, '_destroyed')
                    )
                if notification._uuid in old:
                    notifications.append(old[notification._uuid])
                else:
                    notifications.append(notification)
            self.notifications = notifications
        return super()._process_events(events)

    def _get_properties(self, doc):
        props = super()._get_properties(doc)
        props['root_node'] = '#notistack-container'
        return props

    @classmethod
    def demo(cls, **params):
        """
        Generates a layout which allows demoing the component.
        """
        msg = TextInput(label='Message', value='This is a message', **params)
        duration = NumberInput(label='Duration', value=0, end=10000, **params)
        ntype = Select(
            label='Type', options=['info', 'warning', 'error', 'success', 'custom'],
            value='info', **params
        )
        background = ColorPicker(label='Color', value='#000000', **params)
        button = Button(label='Notify', **params)
        notifications = cls()
        button.js_on_click(
            args={
                'notifications': notifications, 'msg': msg, 'duration': duration,
                'ntype': ntype, 'color': background
            }, code="""
            const config = {
              message: msg.data.value,
              duration: duration.data.value,
              notification_type: ntype.data.value,
              _destroyed: false,
              _uuid: Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15)
            }
            if (ntype.data.value === 'custom') {
              config.background = color.data.value
            }
            notifications.document.event_manager.trigger({event_name: 'esm_event', model: notifications, data: {type: 'enqueue', notification: config}})
            notifications.data.notifications = [...notifications.data.notifications, config]
            """
        )

        return Column(msg, duration, ntype, background, button, notifications)

    def send(self, message, duration=3000, type='default', background=None, icon=None):
        """
        Sends a notification to the frontend.
        """
        notification = self._notification_type(
            message=message, duration=duration, notification_type=type,
            background=background, icon=icon, notification_area=self
        )
        self._notification_watchers[notification] = (
            notification.param.watch(self._remove_notification, '_destroyed')
        )
        self.notifications.append(notification)
        self.param.trigger('notifications')
        self._send_msg({
            'type': 'enqueue',
            'notification': {
                k: v for k, v in notification.param.values().items()
                if k != 'notification_area'
            }
        })
        return notification

    def clear(self):
        for notification in self.notifications:
            notification.param.unwatch(self._notification_watchers.pop(notification))
            self._send_msg({'type': 'destroy', 'uuid': notification._uuid})
        self.notifications = []

    def _handle_msg(self, msg):
        if msg['type'] == 'destroy':
            self.notifications = [n for n in self.notifications if n._uuid != msg['uuid']]

    def _remove_notification(self, event):
        if event.obj in self.notifications:
            self.notifications.remove(event.obj)
        event.obj.param.unwatch(self._notification_watchers.pop(event.obj))
        self.param.trigger('notifications')
        self._send_msg({'type': 'destroy', 'uuid': event.obj._uuid})


_state._notification_type = NotificationArea
_DATA_MODELS[MuiNotification] = construct_data_model(MuiNotification, f'MuiNotification{uuid.uuid4().hex}')



import time

import pytest

pytest.importorskip('playwright')

from panel.tests.util import serve_component, wait_until
from panel_material_ui.widgets.menus import Breadcrumbs, List, MenuButton, Pagination, SpeedDial
from playwright.sync_api import expect

pytestmark = pytest.mark.ui


def test_breadcrumbs(page):
    widget = Breadcrumbs(name='Breadcrumbs test', items=['Home', 'Dashboard', 'Profile'])
    serve_component(page, widget)

    expect(page.locator(".breadcrumbs")).to_have_count(1)
    expect(page.locator(".MuiBreadcrumbs-ol")).to_have_count(1)
    expect(page.locator(".MuiBreadcrumbs-li")).to_have_count(3)
    expect(page.locator(".MuiBreadcrumbs-li").nth(0)).to_have_text("Home")
    expect(page.locator(".MuiBreadcrumbs-li").nth(1)).to_have_text("Dashboard")
    expect(page.locator(".MuiBreadcrumbs-li").nth(2)).to_have_text("Profile")

    for i in range(3):
        page.locator(".MuiBreadcrumbs-li").nth(i).click()
        wait_until(lambda: widget.value == widget.items[i], page)

def test_list(page):
    widget = List(name='List test', items=['Item 1', 'Item 2', 'Item 3'])
    serve_component(page, widget)

    expect(page.locator(".list")).to_have_count(1)
    expect(page.locator(".MuiList-root")).to_have_count(1)

    expect(page.locator(".MuiListItemText-root")).to_have_count(3)
    expect(page.locator(".MuiListItemText-root").nth(0)).to_have_text("Item 1")
    expect(page.locator(".MuiListItemText-root").nth(1)).to_have_text("Item 2")
    expect(page.locator(".MuiListItemText-root").nth(2)).to_have_text("Item 3")

    for i in range(3):
        page.locator(".MuiListItemButton-root").nth(i).click()
        wait_until(lambda: widget.value == widget.items[i], page)

def test_speed_dial(page):
    widget = SpeedDial(name='SpeedDial test', items=[
        {'label': 'Item 1', 'icon': 'home'},
        {'label': 'Item 2', 'icon': 'dashboard'},
        {'label': 'Item 3', 'icon': 'profile'}
    ])
    serve_component(page, widget)

    expect(page.locator(".speed-dial")).to_have_count(1)
    expect(page.locator(".MuiSpeedDial-root")).to_have_count(1)
    expect(page.locator(".MuiSpeedDial-fab")).to_have_count(1)

    for _ in range(3):
        try:
            page.locator(".MuiSpeedDial-fab").hover(force=True)
        except Exception as e:
            time.sleep(0.1)
        else:
            break
    expect(page.locator(".MuiSpeedDial-actions")).to_be_visible()
    expect(page.locator(".MuiSpeedDial-actions button")).to_have_count(3)

    page.locator(".MuiSpeedDial-actions button").nth(0).hover()
    expect(page.locator("#SpeedDialtest-action-0")).to_be_visible()
    expect(page.locator("#SpeedDialtest-action-0")).to_have_text("Item 1")
    page.locator(".MuiSpeedDial-actions button").nth(1).hover()
    expect(page.locator("#SpeedDialtest-action-1")).to_be_visible()
    expect(page.locator("#SpeedDialtest-action-1")).to_have_text("Item 2")
    page.locator(".MuiSpeedDial-actions button").nth(2).hover()
    expect(page.locator("#SpeedDialtest-action-2")).to_be_visible()
    expect(page.locator("#SpeedDialtest-action-2")).to_have_text("Item 3")

    for i in range(3):
        page.locator(".MuiSpeedDial-actions button").nth(i).click()
        wait_until(lambda: widget.value == widget.items[i], page)


def test_breadcrumbs_basic(page):
    items = ['Home', 'Library', 'Data']
    widget = Breadcrumbs(items=items)
    serve_component(page, widget)

    breadcrumbs = page.locator('.MuiBreadcrumbs-ol .MuiBreadcrumbs-li')
    expect(breadcrumbs).to_have_count(3)
    expect(breadcrumbs.nth(2)).to_have_text('Data')

def test_breadcrumbs_click(page):
    events = []
    def cb(event):
        events.append(event)

    items = ['Home', 'Library', 'Data']
    widget = Breadcrumbs(items=items, on_click=cb)
    serve_component(page, widget)

    breadcrumbs = page.locator('.MuiBreadcrumbs-ol .MuiBreadcrumbs-li').nth(1)
    breadcrumbs.click()
    wait_until(lambda: len(events) == 1, page)
    assert events[0] == 'Library'

def test_list_basic(page):
    items = [
        {'label': 'Item 1'},
        {'label': 'Item 2'},
        {'label': 'Item 3'}
    ]
    widget = List(items=items)
    serve_component(page, widget)

    list_items = page.locator('.MuiListItemButton-root')
    expect(list_items).to_have_count(3)

def test_list_nested(page):
    items = [
        {
            'label': 'Item 1',
            'items': [
                {'label': 'Subitem 1'},
                {'label': 'Subitem 2'}
            ]
        }
    ]
    widget = List(items=items)
    serve_component(page, widget)

    # Click to expand
    page.locator('.MuiListItemButton-root').first.click()

    # Check subitems are visible
    subitems = page.locator('.MuiCollapse-root .MuiListItemButton-root')
    expect(subitems).to_have_count(2)

def test_list_selection(page):
    items = [
        {'label': 'Item 1'},
        {'label': 'Item 2'},
        {'label': 'Item 3'}
    ]
    widget = List(items=items)
    serve_component(page, widget)

    list_items = page.locator('.MuiListItemButton-root')
    list_items.nth(1).click()

    assert widget.active == 1
    assert widget.value == items[1]

def test_menubutton_basic(page):
    items = ['Option 1', 'Option 2', 'Option 3']
    widget = MenuButton(items=items, label='Menu')
    serve_component(page, widget)

    # Check button exists
    button = page.locator('.MuiButtonBase-root')
    expect(button).to_have_text('Menu')

    # Click to open menu
    button.click()

    # Check menu items
    menu_items = page.locator('.MuiMenuItem-root')
    expect(menu_items).to_have_count(3)

def test_menubutton_selection(page):
    items = ['Option 1', 'Option 2', 'Option 3']
    widget = MenuButton(items=items, label='Menu')
    serve_component(page, widget)

    # Open menu and select item
    page.locator('.MuiButton-root').click()
    page.locator('.MuiMenuItem-root').nth(1).click()

    assert widget.active == 1
    assert widget.value == 'Option 2'

def test_pagination_basic(page):
    widget = Pagination(count=5)
    serve_component(page, widget)

    pagination = page.locator('.MuiPagination-root')
    expect(pagination).to_have_count(1)

    # Check number of page buttons (including prev/next)
    page_buttons = page.locator('.MuiPaginationItem-root')
    expect(page_buttons).to_have_count(7)  # 5 pages + prev/next buttons

def test_pagination_navigation(page):
    widget = Pagination(count=5)
    serve_component(page, widget)

    # Click second page
    page.locator('.MuiPaginationItem-root').nth(2).click()
    wait_until(lambda: widget.value == 1, page)

def test_speeddial_basic(page):
    items = [
        {'label': 'Copy', 'icon': 'content_copy'},
        {'label': 'Save', 'icon': 'save'},
        {'label': 'Print', 'icon': 'print'}
    ]
    widget = SpeedDial(items=items)
    serve_component(page, widget)

    # Check SpeedDial exists
    speeddial = page.locator('.MuiSpeedDial-root')
    expect(speeddial).to_have_count(1)

    # Click to open
    page.locator('.MuiSpeedDial-fab').click()

    # Check actions are visible
    actions = page.locator('.MuiSpeedDialAction-fab')
    expect(actions).to_have_count(3)

def test_speeddial_selection(page):
    events = []
    def cb(event):
        events.append(event)

    items = [
        {'label': 'Copy', 'icon': 'content_copy'},
        {'label': 'Save', 'icon': 'save'},
        {'label': 'Print', 'icon': 'print'}
    ]
    widget = SpeedDial(items=items, on_click=cb)
    serve_component(page, widget)

    # Open and click an action
    page.locator('.MuiSpeedDial-fab').click()
    page.locator('.MuiSpeedDialAction-fab').nth(1).click()

    wait_until(lambda: len(events) == 1, page)
    assert widget.value == items[1]



import pytest

pytest.importorskip('playwright')

import datetime as dt

from panel.tests.util import serve_component, wait_until
from panel_material_ui.widgets import (
    DatePicker,
    TextInput, PasswordInput, TextAreaInput, Checkbox, Switch, FileInput, ToggleIcon,
)
from playwright.sync_api import expect

pytestmark = pytest.mark.ui


# observe when serving the component
TEXTAREA_LINE_HEIGHT = 23


@pytest.mark.parametrize('variant', ["filled", "outlined", "standard"])
def test_text_input_variant(page, variant):
    widget = TextInput(name='Name', placeholder='Enter your name here ...', variant=variant)
    serve_component(page, widget)
    expect(page.locator('.text-input')).to_have_count(1)
    if variant == "standard":
        expect(page.locator('.MuiInput-root')).to_have_count(1)
    else:
        expect(page.locator(f'.Mui{variant.capitalize()}Input-root')).to_have_count(1)


def test_text_input_typing(page):
    widget = TextInput(name='Test', placeholder='Type something...')
    serve_component(page, widget)

    # Find the input field and type into it
    input_field = page.locator('input').nth(0)
    input_field.click()
    input_field.type('Hello World', delay=50)

    # Check that the text appears as we type (value_input)
    expect(input_field).to_have_value('Hello World')

    # Check value_input is updated while typing
    wait_until(lambda: widget.value_input == 'Hello World', page)

    # But the main value should only be updated when we press Enter
    assert widget.value == ''

    # Press Enter to update the value
    input_field.press('Enter')
    wait_until(lambda: widget.value == 'Hello World', page)

    # Test that typing more updates the display and value_input but not the value
    input_field.type(' Again', delay=50)
    expect(input_field).to_have_value('Hello World Again')
    wait_until(lambda: widget.value_input == 'Hello World Again', page)
    assert widget.value == 'Hello World'

    # Press Enter to update the value
    input_field.press('Enter')
    wait_until(lambda: widget.value == 'Hello World Again', page)


@pytest.mark.from_panel
def test_textinput_enter_pressed(page):
    text_input = TextInput()
    clicks = [0]

    def on_enter(event):
        clicks[0] += 1

    text_input.param.watch(on_enter, "enter_pressed")
    serve_component(page, text_input)

    # Find the input field and type into it
    input_area = page.locator('input').nth(0)
    input_area.click()
    input_area.press('Enter')
    wait_until(lambda: clicks[0] == 1)
    input_area.press("Enter")
    wait_until(lambda: clicks[0] == 2)


def test_textinput_max_length(page):
    widget = TextInput(max_length=2)
    serve_component(page, widget)

    # Find the input field and type into it
    input_area = page.locator('input').nth(0)
    input_area.click()
    # type more but only first max_length characters are allowed
    input_area.type("123")
    expect(input_area).to_have_value("12")
    wait_until(lambda: widget.value_input == "12", page)


def test_password_show_hide(page):
    widget = PasswordInput(label='Password', placeholder='Enter your password here ...')
    serve_component(page, widget)
    expect(page.locator('.password-input')).to_have_count(1)
    expect(page.locator('input[type="password"]')).to_have_count(1)
    # click to show password
    eye_button = page.locator('button[aria-label="display the password"]')
    eye_button.click()
    # password is displayed
    expect(page.locator('input[type="text"]')).to_have_count(1)


def test_password_max_length(page):
    widget = PasswordInput(max_length=2)
    serve_component(page, widget)

    # Find the input field and type into it
    input_area = page.locator('input').nth(0)
    input_area.click()
    # type more but only first max_length characters are allowed
    input_area.type("123")
    expect(input_area).to_have_value("12")
    wait_until(lambda: widget.value_input == "12", page)


def test_text_area_input(page):
    widget = TextAreaInput(label='Description', placeholder='Enter your description here...', rows=5)
    serve_component(page, widget)
    expect(page.locator('.text-area-input')).to_have_count(1)
    expect(page.locator('textarea[rows="5"]')).to_have_count(1)


def test_text_area_typing(page):
    widget = TextAreaInput(label='Description', placeholder='Type something...')
    serve_component(page, widget)

    # Find the textarea and type into it
    textarea = page.locator('textarea').nth(0)
    textarea.click()

    # Type text including newlines
    textarea.type('Multiline', delay=50)
    textarea.press('Enter')
    textarea.type('Text', delay=50)
    textarea.press('Enter')
    textarea.type('Test', delay=50)

    # Check that the text appears as we type (value_input)
    expect(textarea).to_have_value('Multiline\nText\nTest')

    # Check value_input is updated while typing
    wait_until(lambda: widget.value_input == 'Multiline\nText\nTest', page)

    # But value should still be the original value (empty string) since we haven't blurred
    assert widget.value == ''

    # Click elsewhere to trigger blur
    page.locator('body').click()
    wait_until(lambda: widget.value == 'Multiline\nText\nTest', page)


@pytest.mark.from_panel
def test_text_area_auto_grow(page):
    widget = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n")
    serve_component(page, widget)

    input_area = page.locator('.MuiInputBase-input').nth(0)
    input_area.click()
    input_area.press('Enter')
    input_area.press('Enter')
    input_area.press('Enter')

    # 8 rows
    wait_until(lambda: input_area.bounding_box()['height'] == 8 * TEXTAREA_LINE_HEIGHT, page)


@pytest.mark.from_panel
def test_text_area_auto_grow_max_rows(page):
    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", max_rows=7)

    serve_component(page, text_area)

    input_area = page.locator('.MuiInputBase-input').nth(0)
    input_area.click()
    input_area.press('Enter')
    input_area.press('Enter')
    input_area.press('Enter')

    wait_until(lambda: input_area.bounding_box()['height'] == 7 * TEXTAREA_LINE_HEIGHT, page)


@pytest.mark.from_panel
def test_text_area_auto_grow_min_rows(page):
    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", rows=3)
    serve_component(page, text_area)

    input_area = page.locator('.MuiInputBase-input').nth(0)
    input_area.click()
    for _ in range(5):
        input_area.press('ArrowDown')
    for _ in range(10):
        input_area.press('Backspace')

    wait_until(lambda: input_area.bounding_box()['height'] == 3 * TEXTAREA_LINE_HEIGHT, page)


@pytest.mark.from_panel
def test_text_area_auto_grow_shrink_back_on_new_value(page):
    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", max_rows=5)
    serve_component(page, text_area)

    input_area = page.locator('.MuiInputBase-input').nth(0)
    input_area.click()
    for _ in range(5):
        input_area.press('ArrowDown')
    for _ in range(10):
        input_area.press('Backspace')

    text_area.value = ""
    assert input_area.bounding_box()['height'] == 2 * TEXTAREA_LINE_HEIGHT


def test_text_area_max_length(page):
    widget = TextAreaInput(max_length=2)
    serve_component(page, widget)

    # Find the input field and type into it
    input_area = page.locator('.MuiInputBase-input').nth(0)
    input_area.click()
    # type more but only first max_length characters are allowed
    input_area.type("123")
    expect(input_area).to_have_value("12")
    wait_until(lambda: widget.value_input == "12", page)


def test_checkbox(page):
    widget = Checkbox(label='Works with the tools you know and love', value=True)
    serve_component(page, widget)
    expect(page.locator('.checkbox')).to_have_count(1)


def test_switch(page):
    widget = Switch(label='Works with the tools you know and love', value=True)
    serve_component(page, widget)
    expect(page.locator('.switch')).to_have_count(1)


def test_fileinput(page):
    widget = FileInput(accept='.png,.jpeg', multiple=True)
    serve_component(page, widget)
    expect(page.locator('.file-input')).to_have_count(1)


def test_toggle_icon(page):
    widget = ToggleIcon(icon="thumb-up", active_icon="thumb-down", size="small", description="Like")
    serve_component(page, widget)

    expect(page.locator('.toggle-icon')).to_have_count(1)
    icon = page.locator('.MuiCheckbox-root')
    expect(icon).to_have_text("thumb-up")
    icon.click()
    expect(icon).to_have_text("thumb-down")


def test_datepicker_enabled_dates(page):
    widget = DatePicker(
        name='Date Picker',
        start=dt.date(2024, 4, 1),
        end=dt.date(2024, 4, 20),
        enabled_dates=[
            dt.date(2024, 4, 1),
            dt.date(2024, 4, 2),
            dt.date(2024, 4, 20),
            dt.date(2024, 4, 21),
        ]
    )
    serve_component(page, widget)

    # click the datepicker icon to show dates to select
    icon = page.locator(".MuiIconButton-root")
    icon.click()
    # Select all buttons in the date picker
    buttons = page.locator('.MuiPickersDay-root').all()

    enabled_buttons = []
    for button in buttons:
        is_disabled = button.get_attribute("disabled") is not None or button.evaluate(
            "el => el.classList.contains('Mui-disabled')")
        if not is_disabled:
            value = button.inner_text()  # Get the button's displayed text (date)
            enabled_buttons.append(value)

    # only enabled dates within the start to end range are selectable
    assert enabled_buttons == ['1', '2', '20']


def test_datepicker_disabled_dates(page):
    widget = DatePicker(
        name='Date Picker',
        start=dt.date(2024, 4, 1),
        end=dt.date(2024, 4, 20),
        disabled_dates=[
            dt.date(2024, 4, i) for i in range(1, 18)
        ]
    )
    serve_component(page, widget)

    # click the datepicker icon to show dates to select
    icon = page.locator(".MuiIconButton-root")
    icon.click()
    # Select all buttons in the date picker
    buttons = page.locator('.MuiPickersDay-root').all()

    enabled_buttons = []
    for button in buttons:
        is_disabled = button.get_attribute("disabled") is not None or button.evaluate(
            "el => el.classList.contains('Mui-disabled')")
        if not is_disabled:
            value = button.inner_text()  # Get the button's displayed text (date)
            enabled_buttons.append(value)

    # only enabled dates within the start to end range are selectable
    assert enabled_buttons == ['18', '19', '20']



import pytest

pytest.importorskip("playwright")

from playwright.sync_api import expect

from panel_material_ui.notifications import NotificationArea
from panel.config import config
from panel.io.state import state
from panel.layout import Row
from panel.pane import Markdown
from panel.tests.util import serve_component, wait_until
from panel.widgets import Button

pytestmark = pytest.mark.ui


def test_notifications(page):
    def callback(event):
        state.notifications.error('MyError')

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Display error')
        button.on_click(callback)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_text('MyError')


def test_notifications_clear(page):
    def callback(event):
        state.notifications.error('MyError', duration=0)
        state.notifications.info('MyInfo', duration=0)

    notifications = []

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Display error')
        button.on_click(callback)
        notifications.append(state.notifications)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_count(2)

    notifications[0].clear()

    expect(page.locator('.MuiAlert-message')).to_have_count(0)


def test_notifications_destroy(page):

    notifications = []

    def add_notifications(event):
        notifications.extend([
            state.notifications.error('MyError', duration=0),
            state.notifications.info('MyInfo', duration=0),
        ])

    def destroy_notifications(event):
        notifications.pop(0).destroy()

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        add = Button(name='Add')
        remove = Button(name='Remove')
        add.on_click(add_notifications)
        remove.on_click(destroy_notifications)
        return Row(add, remove)

    serve_component(page, app)

    # Add and destroy two notifications
    messages = page.locator('.MuiAlert-message')
    page.locator('.bk-btn').nth(0).click()
    expect(messages).to_have_count(2)
    page.locator('.bk-btn').nth(1).click()
    expect(messages).to_have_count(1)
    page.locator('.bk-btn').nth(1).click()
    expect(messages).to_have_count(0)


def test_notifications_custom_background(page):
    def callback(event):
        state.notifications.send('Custom notification', background='#000000', duration=0)

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Display error')
        button.on_click(callback)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_text('Custom notification')
    expect(page.locator('.MuiPaper-root')).to_have_css('background-color', 'rgb(0, 0, 0)')


def test_notifications_custom_type(page):
    def callback(event):
        state.notifications.send('Custom notification', type='custom', duration=0)

    def app():
        config.notifications = True
        state.notifications.types = [{'type': 'custom', 'background': '#000000', 'icon': 'home'}]
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Display error')
        button.on_click(callback)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_text('Custom notification')
    expect(page.locator('.MuiPaper-root')).to_have_css('background-color', 'rgb(0, 0, 0)')
    expect(page.locator('.MuiAlert-icon')).to_have_text('home')


def test_notifications_dismiss(page):
    def callback(event):
        state.notifications.error('MyError', duration=0)

    notifications = []

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Display error')
        button.on_click(callback)
        notifications.append(state.notifications)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_text('MyError')

    page.click('.MuiIconButton-root')

    expect(page.locator('.MuiAlert-message')).to_be_hidden()

    wait_until(lambda: len(notifications[0].notifications) == 0, page)


def test_ready_notification(page):
    def app():
        config.ready_notification = 'Ready!'
        assert isinstance(state.notifications, NotificationArea)
        return Markdown('Ready app')

    serve_component(page, app)

    expect(page.locator('.MuiAlert-message')).to_have_text('Ready!')


def test_disconnect_notification(page):
    def app():
        config.disconnect_notification = 'Disconnected!'
        assert isinstance(state.notifications, NotificationArea)
        button = Button(name='Stop server')
        button.js_on_click(code="""
        Bokeh.documents[0].event_manager.send_event({'event_name': 'connection_lost', 'publish': false})
        """)
        return button

    serve_component(page, app)

    page.click('.bk-btn')

    expect(page.locator('.MuiAlert-message')).to_have_text('Disconnected!')


def test_onload_notification(page):
    def onload_callback():
        state.notifications.warning("Warning", duration=0)
        state.notifications.info("Info", duration=0)

    def app():
        config.notifications = True
        assert isinstance(state.notifications, NotificationArea)
        state.onload(onload_callback)
        return Markdown("# Hello world")

    serve_component(page, app)

    expect(page.locator('.MuiAlert-message')).to_have_count(2)
    expect(page.locator('.MuiAlert-colorWarning .MuiAlert-message')).to_have_text("Warning")
    expect(page.locator('.MuiAlert-colorInfo .MuiAlert-message')).to_have_text("Info")



"""
Material UI theme configuration for the Orbitron brand.

This module provides functions to configure Panel Material UI components
with consistent theming, typography, and styling according to brand guidelines.
"""
import panel_material_ui as pmui
import panel as pn
from typing import Dict, Any
from copy import deepcopy

from .colors import DARK_THEME, LIGHT_THEME
from .assets import FAVICON_PATH, LOGO_PATH, RAW_CSS

# Theme configuration for light mode
_LIGHT_THEME_CONFIG: Dict[str, Any] = {
    "palette": {
        "primary": {
            "main": LIGHT_THEME.primary,
        },
        "secondary": {"main": LIGHT_THEME.secondary},
        "success": {"main": LIGHT_THEME.success},
        "error": {"main": LIGHT_THEME.error},
        "warning": {"main": LIGHT_THEME.warning},
        "info": {"main": LIGHT_THEME.info},
        "contrastThreshold": 3,
        "tonalOffset": 0.2,
    },
    "typography": {
        "fontFamily": ("Montserrat", "Helvetica Neue", "Arial", "sans-serif"),
        "fontSize": 16,
        "fontWeight": 700,
        "letterSpacing": 10.2,
        "lineHeight": 1.5,
    },
    "shape": {
        "borderRadius": 8,
    },
    "components": {
        "MuiButtonBase": {
            "defaultProps": {
                "disableRipple": True,
            },
        },
    },
}

# Create dark theme configuration from light theme
_DARK_THEME_CONFIG = deepcopy(_LIGHT_THEME_CONFIG)
_DARK_THEME_CONFIG["palette"]["primary"] = {
    "main": DARK_THEME.primary,
}
_DARK_THEME_CONFIG["palette"]["secondary"] = {
    "main": DARK_THEME.secondary,
}

# Combined theme configuration
THEME_CONFIG: Dict[str, Dict[str, Any]] = {
    "light": _LIGHT_THEME_CONFIG,
    "dark": _DARK_THEME_CONFIG,
}

# Custom notification message
_DISCONNECT_NOTIFICATION: str = """The connection to the server was lost. Please refresh to \
reconnect."""

# Montserrat font URL
_MONTSERRAT_FONT_URL: str = "https://fonts.googleapis.com/css2?family=Montserrat:ital,wght@0,100..900;1,100..900&display=swap"


def _configure_session() -> None:
    """
    Configure Panel session-specific settings.

    This includes disconnect notifications and other session-level configurations.
    """
    pn.config.disconnect_notification = _DISCONNECT_NOTIFICATION


@pn.cache
def _configure_general() -> None:
    """
    Configure general theme settings for Panel Material UI components.

    This includes theme configuration, CSS, fonts, logos, and component defaults.
    """
    # Page configuration
    pmui.Page.param.theme_config.default = THEME_CONFIG

    # CSS and font configuration
    if RAW_CSS:
        pmui.Page.config.raw_css.append(RAW_CSS)

    pmui.Page.config.css_files.append(_MONTSERRAT_FONT_URL)

    # Brand assets configuration
    pmui.Page.param.logo.default = LOGO_PATH
    pmui.Page.favicon = FAVICON_PATH
    pmui.Page.meta.apple_touch_icon = ""  # Intentionally left empty
    pmui.Page.meta.name = "Orbitron"

    # Component-specific configurations
    pmui.Button.param.disable_elevation.default = True

    # Fix missing closing bracket in stylesheets
    pn.pane.Image.stylesheets = ["img {border-radius: 2px}"]
    pn.widgets.Tabulator.param.theme = "materialize"


def configure() -> None:
    """
    Configure the complete theme for the application.

    This is the main entry point for applying the Orbitron brand theme
    to a Panel Material UI application.

    Examples
    --------
    >>> from brand.mui import configure
    >>> configure()
    >>> app = pmui.Page(title="My Orbitron App")
    """
    _configure_general()
    _configure_session()


# If module is run directly, apply configuration
if __name__ == "__main__":
    configure()



import json
import os
import pathlib

from typing import Any

import param

param.parameterized.docstring_signature = False
param.parameterized.docstring_describe_params = False

from nbsite.shared_conf import *

project = 'Panel Material UI'
authors = 'Panel contributors'
copyright_years['start_year'] = '2024'
copyright = copyright_fmt.format(**copyright_years)
description = 'Panel Material UI extension'

import panel
import panel_material_ui

from panel.io.convert import (
    BOKEH_VERSION, MINIMUM_VERSIONS, PY_VERSION, PYODIDE_VERSION,
    PYSCRIPT_VERSION,
)
from panel.io.resources import CDN_ROOT

PANEL_ROOT = pathlib.Path(panel.__file__).parent

version = release = base_version(panel_material_ui.__version__)
js_version = json.loads((PANEL_ROOT / 'package.json').read_text())['version']

is_dev = any(ext in version for ext in ('a', 'b', 'rc'))

# For the interactivity warning box created by nbsite to point to the right
# git tag instead of the default i.e. main.
os.environ['BRANCH'] = f"v{release}"

html_static_path += ['_static']

html_css_files += [
    'css/custom.css',
]

html_theme = "pydata_sphinx_theme"
html_favicon = "_static/icons/favicon.ico"

current_release = panel_material_ui.__version__  # Current release version variable

html_theme_options = {
    "logo": {
        "image_light": "_static/logo_horizontal_light_theme.png",
        "image_dark": "_static/logo_horizontal_dark_theme.png",
    },
    "github_url": "https://github.com/panel-extensions/panel-material-ui",
    "icon_links": [
        {
            "name": "Twitter",
            "url": "https://twitter.com/Panel_Org",
            "icon": "fa-brands fa-twitter-square",
        },
        {
            "name": "Discourse",
            "url": "https://discourse.holoviz.org/c/panel/5",
            "icon": "fa-brands fa-discourse",
        },
        {
            "name": "Discord",
            "url": "https://discord.gg/UXdtYyGVQX",
            "icon": "fa-brands fa-discord",
        },
    ],
    "pygments_light_style": "material",
    "pygments_dark_style": "material",
    "header_links_before_dropdown": 5,
    'secondary_sidebar_items': [
        # "github-stars-button",
        # "panelitelink",
        "page-toc",
    ],
}


extensions = [
    'numpydoc',
    'bokeh.sphinxext.bokeh_plot',
    'myst_parser',
    'sphinx_design',
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.ifconfig',
    'sphinx.ext.linkcode',
    'sphinx.ext.inheritance_diagram',
    'sphinx_copybutton',
    'sphinxext.rediraffe',
    'nbsite.gallery',
    'nbsite.pyodide',
    'nbsite.analytics',
]

numpydoc_show_inherited_class_members = False
numpydoc_class_members_toctree = False
napoleon_numpy_docstring = True

autodoc_mock_imports = ["panel.pane.vtk"]

myst_enable_extensions = ["colon_fence", "deflist"]

gallery_endpoint = 'panel-gallery-dev' if is_dev else 'panel-gallery'
gallery_url = f'https://{gallery_endpoint}.holoviz-demo.anaconda.com'
jlite_url = 'https://holoviz-dev.github.io/panelite-dev/lab' if is_dev else 'https://panelite.holoviz.org/lab'
pyodide_url = 'https://holoviz-dev.github.io/panel/pyodide' if is_dev else 'https://panel.holoviz.org/pyodide'

nbsite_analytics = {
    'goatcounter_holoviz': True,
}

nbsite_gallery_conf = {
    'github_org': 'panel-extensions',
    'github_project': 'panel-material-ui',
    'galleries': {
        'reference': {
            'title': 'Component Gallery',
            'extensions': ['*.ipynb', '*.py', '*.md'],
            'sections': [
                'widgets',
                'menus',
                'layouts',
                'page',
                'chat',
                'indicators',
                'global'
            ],
            'as_pyodide': True,
            'normalize_titles': False,
        }
    },
    'thumbnail_url': 'https://assets.holoviz.org/panel-material-ui/thumbnails',
    'only_use_existing': True,
}

if panel.__version__ != version and (PANEL_ROOT / 'dist' / 'wheels').is_dir():
    py_version = panel.__version__.replace("-dirty", "")
    panel_req = f'./wheels/panel-{py_version}-py3-none-any.whl'
    bokeh_req = f'./wheels/bokeh-{BOKEH_VERSION}-py3-none-any.whl'
else:
    panel_req = f'{CDN_ROOT}wheels/panel-{PY_VERSION}-py3-none-any.whl'
    bokeh_req = f'{CDN_ROOT}wheels/bokeh-{BOKEH_VERSION}-py3-none-any.whl'

html_js_files = [
    (None, {'body': '{"shimMode": true}', 'type': 'esms-options'}),
    f'https://cdn.holoviz.org/panel/{js_version}/dist/bundled/reactiveesm/es-module-shims@^1.10.0/dist/es-module-shims.min.js',
]

nbsite_pyodide_conf = {
    'PYODIDE_URL': f'https://cdn.jsdelivr.net/pyodide/{PYODIDE_VERSION}/full/pyodide.js',
    'requirements': [bokeh_req, panel_req, 'pyodide-http'],
    'requires': ["panel-material-ui"],
}

templates_path += [
    '_templates'
]

html_context.update({
    "last_release": f"v{release}",
    "github_user": "panel-extensions",
    "github_repo": "panel-material-ui",
    "default_mode": "light",
    "panelite_endpoint": jlite_url,
    "gallery_url": gallery_url,
    "pyodide_url": pyodide_url
})

nbbuild_patterns_to_take_along = ["simple.html", "*.json", "json_*"]

# Override the Sphinx default title that appends `documentation`
html_title = f'{project} v{version}'

# Ensure the global version string includes the SHA to ensure the service
# worker is invalidated on builds between tags
if is_dev:
    version = panel.__version__

# # Patching GridItemCardDirective to be able to substitute the domain name
# # in the link option.
# from sphinx_design.cards import CardDirective
# from sphinx_design.grids import GridItemCardDirective

# orig_grid_run = GridItemCardDirective.run

# def patched_grid_run(self):
#     app = self.state.document.settings.env.app
#     existing_link = self.options.get('link')
#     domain = getattr(app.config, 'grid_item_link_domain', None)
#     if self.has_content:
#         self.content.replace('|gallery-endpoint|', domain)
#     if existing_link and domain:
#         new_link = existing_link.replace('|gallery-endpoint|', domain)
#         self.options['link'] = new_link
#     return list(orig_grid_run(self))

# GridItemCardDirective.run = patched_grid_run

# orig_card_run = CardDirective.run

# def patched_card_run(self):
#     app = self.state.document.settings.env.app
#     existing_link = self.options.get('link')
#     domain = getattr(app.config, 'grid_item_link_domain', None)
#     if existing_link and domain:
#         new_link = existing_link.replace('|gallery-endpoint|', domain)
#         self.options['link'] = new_link
#     return orig_card_run(self)

# CardDirective.run = patched_card_run

def _get_pyodide_version():
    if PYODIDE_VERSION.startswith("v"):
        return PYODIDE_VERSION[1:]
    raise NotImplementedError(F"{PYODIDE_VERSION=} is not valid")

def update_versions(app, docname, source):
    # Inspired by: https://stackoverflow.com/questions/8821511
    version_replace = {
        "{{PANEL_VERSION}}" : PY_VERSION,
        "{{BOKEH_VERSION}}" : BOKEH_VERSION,
        "{{PYSCRIPT_VERSION}}" : PYSCRIPT_VERSION,
        "{{PYODIDE_VERSION}}" : _get_pyodide_version()
    }

    for old, new in version_replace.items():
        source[0] = source[0].replace(old, new)


def setup_mystnb(app):
    from myst_nb.core.config import NbParserConfig
    from myst_nb.sphinx_ import (
        HideCodeCellNode, HideInputCells, SelectMimeType,
    )
    from myst_nb.sphinx_ext import create_mystnb_config

    _UNSET = "--unset--"
    for name, default, field in NbParserConfig().as_triple():
        if not field.metadata.get("sphinx_exclude"):
            # TODO add types?
            app.add_config_value(f"nb_{name}", default, "env", Any)  # type: ignore[arg-type]
            if "legacy_name" in field.metadata:
                app.add_config_value(
                    f"{field.metadata['legacy_name']}",
                    _UNSET,
                    "env",
                    Any,  # type: ignore[arg-type]
                )
    app.add_config_value("nb_render_priority", _UNSET, "env", Any)  # type: ignore[arg-type]
    create_mystnb_config(app)

    # add post-transform for selecting mime type from a bundle
    app.add_post_transform(SelectMimeType)

    # setup collapsible content
    app.add_post_transform(HideInputCells)
    HideCodeCellNode.add_to_app(app)


def generate_demo(app):
    subprocess.run(["python", pathlib.Path(__file__).parent / "_scripts" / "demo.py"], check=True)

def setup(app) -> None:
    try:
        from nbsite.paramdoc import param_formatter, param_skip
        app.connect('autodoc-process-docstring', param_formatter)
        app.connect('autodoc-skip-member', param_skip)
    except ImportError:
        print('no param_formatter (no param?)')

    app.connect('builder-inited', setup_mystnb)
    app.connect("builder-inited", generate_demo)

    app.connect('source-read', update_versions)
    nbbuild.setup(app)
    app.add_config_value('grid_item_link_domain', '', 'html')

    # hv_sidebar_dropdown
    app.add_config_value('nbsite_hv_sidebar_dropdown', {}, 'html')
    app.connect("html-page-context", add_hv_sidebar_dropdown_context)


grid_item_link_domain = gallery_endpoint



from panel.chat.feed import ChatFeed as _PnChatFeed
from panel.layout import Column

from ..layout import Card
from .message import ChatMessage
from .step import ChatStep


class ChatFeed(_PnChatFeed):
    """
    A ChatFeed holds a list of `ChatMessage` objects and provides convenient APIs.
    to interact with them.

    This includes methods to:
    - Send (append) messages to the chat log.
    - Stream tokens to the latest `ChatMessage` in the chat log.
    - Execute callbacks when a user sends a message.
    - Undo a number of sent `ChatMessage` objects.
    - Clear the chat log of all `ChatMessage` objects.

    :References:

    - https://panel-material-ui.holoviz.org/reference/chat/ChatFeed.html
    - https://panel.holoviz.org/reference/chat/ChatFeed.html

    :Example:

    >>> async def say_welcome(contents, user, instance):
    >>>    yield "Welcome!"
    >>>    yield "Glad you're here!"

    >>> chat_feed = ChatFeed(callback=say_welcome, header="Welcome Feed")
    >>> chat_feed.send("Hello World!", user="New User", avatar="😊")
    """
    _card_type = Card
    _message_type = ChatMessage
    _step_type = ChatStep

    def _build_steps_layout(self, step, layout_params, default_layout):
        layout_params = layout_params or {}
        input_layout_params = dict(
            min_width=100
        )
        if default_layout == "column":
            layout = Column
        elif default_layout == "card":
            layout = self._card_type
            title = layout_params.pop("title", None)
            input_layout_params["title"] = title or "🪜 Steps"
            input_layout_params["sizing_mode"] = "stretch_width"
        else:
            raise ValueError(
                f"Invalid default_layout {default_layout!r}; "
                f"expected 'column' or 'card'."
            )
        if layout_params:
            input_layout_params.update(layout_params)
        return layout(step, **input_layout_params)

__all__ = ["ChatFeed"]



import pytest

pytest.importorskip('playwright')

from panel.tests.util import serve_component, wait_until
from panel_material_ui.widgets import Button, ButtonIcon, Toggle
from playwright.sync_api import expect

pytestmark = pytest.mark.ui


@pytest.mark.parametrize('button_style', ['contained', 'outlined', 'text'])
def test_button_style(page, button_style):
    widget = Button(name='Click', button_style=button_style, button_type='primary')
    serve_component(page, widget)
    button_format = page.locator(f'.MuiButton-{button_style}Primary')
    expect(button_format).to_have_count(1)


@pytest.mark.parametrize('button_type', ['primary', 'secondary', 'error', 'info', 'success', 'warning'])
def test_button_type(page, button_type):
    widget = Button(name='Click', button_style='contained', button_type=button_type)
    serve_component(page, widget)
    button_format = page.locator(f'.MuiButton-contained{button_type.capitalize()}')
    expect(button_format).to_have_count(1)


def test_button_on_click(page):
    events = []
    def cb(event):
        events.append(event)

    widget = Button(name='Click', on_click=cb)
    serve_component(page, widget)
    button = page.locator('.button')
    expect(button).to_have_count(1)
    button.click()
    wait_until(lambda: len(events) == 1, page)


def test_button_handle_click(page):
    widget = Button(name='Click')
    assert widget.clicks == 0
    serve_component(page, widget)
    button = page.locator('.button')
    button.click()
    assert widget.clicks == 1


def test_button_icon(page):
    widget = ButtonIcon(
        icon='favorite',
        active_icon='check',
        toggle_duration=3000,
    )
    serve_component(page, widget)
    button_icon = page.locator('.button-icon')
    icon = page.locator('.material-icons')

    expect(button_icon).to_have_count(1)
    expect(icon).to_have_text('favorite')

    button_icon.click(force=True)
    expect(icon).to_have_text('check')


@pytest.mark.parametrize('button_type', ['primary', 'secondary', 'error', 'info', 'success', 'warning'])
def test_button_icon_format(page, button_type):
    widget = ButtonIcon(
        icon='favorite',
        active_icon='check',
        button_type=button_type,
        toggle_duration=3000,
    )
    serve_component(page, widget)
    button_color = page.locator(f'.MuiIconButton-color{button_type.capitalize()}')
    expect(button_color).to_have_count(1)


def test_toggle(page):
    widget = Toggle()
    serve_component(page, widget)
    assert widget.value == False  # noqa
    toggle = page.locator('.toggle')
    expect(toggle).to_have_count(1)
    toggle_nonpressed = page.locator("button[aria-pressed='false']")
    expect(toggle_nonpressed).to_have_count(1)

    toggle.click()
    toggle_pressed = page.locator("button[aria-pressed='true']")
    expect(toggle_pressed).to_have_count(1)
    wait_until(lambda: widget.value, page=page)


@pytest.mark.parametrize('button_type', ['primary', 'secondary', 'error', 'info', 'success', 'warning'])
def test_toggle_format(page, button_type):
    widget = Toggle(button_type=button_type)
    serve_component(page, widget)
    if button_type == 'error':
        button_color = page.locator(f'.Mui-{button_type}')
    else:
        button_color = page.locator(f'.MuiToggleButton-{button_type}')
    expect(button_color).to_have_count(1)
